// Authors: Sven Förster, Duc Thanh Tran

#include "equitableColoring.hpp"

#include <cstdlib>
#include <iostream>

void outputProgramHelp() {
	std::cout << "Usage: ./eqDsatur [N | R] [N | C] nPruningRule timeLimit"
	<< " threshold [FILE | n p SEED] ClPerc"
	<< " NoColorExclColor NoColorCard NoColorExclColorClq doMaxFlow nPruningComb nCliqueDecomp (starting_LB starting_UB)\n";
}

bool readInputArgs(const int argc, char* argv[], EqDsatur::Parameters &p) {
	if(argc < 2) {
		std::cout << "Missing arguments\n";
		return false;
	}

	char *end;
	if (std::string(argv[1]) == "N" && (argc == 14 || argc == 16)) {
		p.eqDsaturVariant = *argv[2];
		p.nPruningRule = static_cast<int>(std::strtol(argv[3], &end, 10));
		p.timeLimit = static_cast<int>(std::strtol(argv[4], &end, 10));
		p.threshold = static_cast<int>(std::strtol(argv[5], &end, 10));
		p.graphFile = argv[6];
		p.pNewCl = std::strtof(argv[7], &end);
		p.enableCombPruning_1 = static_cast<int>(std::strtol(argv[8], &end, 10));
		p.enableCombPruning_2 = static_cast<int>(std::strtol(argv[9], &end, 10));
		p.enableCombPruning_3 = static_cast<int>(std::strtol(argv[10], &end, 10));
		p.doMaxFlowPruning = static_cast<bool>(std::strtol(argv[11], &end, 10));
		p.nPruningCombRule = static_cast<int>(std::strtol(argv[12], &end, 10));
		p.nCliqueDecompositionRule = static_cast<int>(std::strtol(argv[13], &end, 10));
		p.graphInput = 'N';
		p.n = 0;
		p.p = 0;
		p.randomSeed = 0;
	} else if (std::string(argv[1]) == "R" && (argc == 16 || argc == 18)) {
		p.eqDsaturVariant = *argv[2];
		p.nPruningRule = static_cast<int>(std::strtol(argv[3], &end, 10));
		p.timeLimit = static_cast<int>(std::strtol(argv[4], &end, 10));
		p.threshold = static_cast<int>(std::strtol(argv[5], &end, 10));
		p.n = static_cast<int>(std::strtol(argv[6], &end, 10));
		p.p = std::strtof(argv[7], &end);
		p.randomSeed = static_cast<int>(std::strtol(argv[8], &end, 10));
		p.pNewCl = std::strtof(argv[9], &end);
		p.enableCombPruning_1 = static_cast<int>(std::strtol(argv[10], &end, 10));
		p.enableCombPruning_2 = static_cast<int>(std::strtol(argv[11], &end, 10));
		p.enableCombPruning_3 = static_cast<int>(std::strtol(argv[12], &end, 10));
		p.doMaxFlowPruning = static_cast<bool>(std::strtol(argv[13], &end, 10));
		p.nPruningCombRule = static_cast<int>(std::strtol(argv[14], &end, 10));
		p.nCliqueDecompositionRule = static_cast<int>(std::strtol(argv[15], &end, 10));
		p.graphInput = 'R';
	} else {
		std::cout << "Not a valid Mode or missing arguments\n";
		return false;
	}
	return true;
}

int main(int argc, char** argv) {
	EqDsatur::Parameters p;
	if (!readInputArgs(argc, argv, p)) {
		outputProgramHelp();
		return EXIT_FAILURE;
	}

	if ((argc == 14 && std::string(argv[1]) == "N") || (argc == 16 && std::string(argv[1]) == "R")) {
		EqDsatur::EquitableColoring ec(p);
	}
	else {
		// NOTE: procedure for bound-input (initial ones) - needs to be adjusted in the future

		// auto lowerBound = 0;
		// auto upperBound = 0;
		//
		// char *end;
		// if (argc == 16) {
		// 	lowerBound = static_cast<int>(std::strtol(argv[12], &end, 10));
		// 	upperBound = static_cast<int>(std::strtol(argv[13], &end, 10));
		// } else if (argc == 18) {
		// 	lowerBound = static_cast<int>(std::strtol(argv[14], &end, 10));
		// 	upperBound = static_cast<int>(std::strtol(argv[15], &end, 10));
		// }
		//
		// if (lowerBound == upperBound) {
		// 	std::cout << "Starting bounds are equal. Exiting\n";
		// 	return EXIT_FAILURE;
		// }
		//
		// EqColoring eqColoring(p, lowerBound, upperBound);
	}
	return EXIT_SUCCESS;
}
