// Authors: Sven Förster, Duc Thanh Tran

#include "randomGraphFactory.hpp"
#include "boost.hpp"

#include <random>
#include <tuple>

namespace EqDsatur {

Graph RandomGraphFactory::create(const int n, const double p, const int seed) noexcept {
	if (seed == 0) {
		std::random_device r;
		std::seed_seq seedSeq {r(), r(), r(), r(), r(), r(), r()};
		engine.seed(seedSeq);
	}
	else {
		engine.seed(seed);
	}
	std::uniform_real_distribution<double> uniform_dist(0, 1);

	Graph graph(n);

	VertexIt vertex, v_end;
	for (std::tie(vertex, v_end) = boost::vertices(graph); vertex != v_end; ++vertex) {
		for (auto vertex2 = vertex+1; vertex2 != v_end; ++vertex2) {
			if (*vertex != *vertex2) {
				const auto randomD = uniform_dist(engine);
				if (randomD <= p) {
					boost::add_edge(*vertex, *vertex2, graph);
				}
			}
		}
	}
	return graph;
}

std::mt19937 RandomGraphFactory::engine;

} // namespace EqDsatur
