// Authors: Sven Förster, Duc Thanh Tran

#ifndef RANDOMGRAPHFACTORY_HPP
#define RANDOMGRAPHFACTORY_HPP

#include "boost.hpp"

#include <random>

namespace EqDsatur {

class RandomGraphFactory {
public:
	RandomGraphFactory() = delete;
	///
	/// @brief Generates random graph
	///
	/// Generates a random, undirected graph with exactly \b n vertices.
	/// Self-loops will not be constructed.
	///
	/// @param[in] n number of vertices
	/// @param[in] p probability to construct an edge between two vertices
	/// @param[in] seed seed value for the random generator. If it is 0 then the random generator
	/// (Mersenne Twister) will be initialized with a seed sequence, which consists of 7 randomized values.
	/// @return Graph
	static Graph create(const int n, const double p, const int seed) noexcept;

private:
	static std::mt19937 engine;
};

} // namespace EqDsatur

#endif
